<?php
/*
Plugin Name: Auto Video Play & Stop
Description: 動画（videoタグ）が挿入されている投稿や固定ページのフッターに、スクロール位置に基づいた動画の自動再生と自動停止を制御するためのJavascriptを挿入します。※controls属性のついた動画は自動再生されません。※動画制御のため、使用テーマ内で別途Jquery（3.6.0以上推奨）を読み込ませておく必要があります。
Version: 1.3
Author: KANAERU WEB
Author URI: https://kanaeru-web.biz/
License: GPL2
*/

// 前面のみでスクリプトを読み込む
function avps_enqueue_scripts() {
  if (!is_admin()) {
    wp_enqueue_script('jquery');

    // JavaScriptコードの直接挿入
    if (is_singular() && (strpos(get_post()->post_content, '<video') !== false)) {
      add_action('wp_footer', 'avps_insert_script', 9999);
    }
  }
}
add_action('wp_enqueue_scripts', 'avps_enqueue_scripts');

function avps_insert_script() {
  $settings = get_option('avps_settings');
  $visible_height = isset($settings['avps_visible_height']) ? $settings['avps_visible_height'] : 50;
  $visible_offset = isset($settings['avps_visible_offset']) ? $settings['avps_visible_offset'] : 50;
  ?>
  <script>
  jQuery(function($) {
    var $window = $(window);
    var $autoplayVideos = $("video:not([controls])");
    var isInitialized = false;

    function handleVideoPlayback() {
      var windowScrollTop = $window.scrollTop();
      var windowInnerHeight = window.innerHeight;

      $autoplayVideos.each(function() {
        var $video = $(this);
        var videoTop = $video.offset().top;
        var videoHeight = $video.innerHeight();
        var videoVisibleHeight = videoHeight * <?php echo intval($visible_height) / 100; ?>;
        var videoVisibleOffset = videoHeight * <?php echo intval($visible_offset) / 100; ?>;
        var videoBottom = videoTop + videoHeight;

        var isTopInViewport = (videoTop + videoVisibleHeight) < (windowScrollTop + windowInnerHeight);
        var isBottomInViewport = (videoBottom - videoVisibleHeight) > windowScrollTop;

        if (!$video.prop("paused") && (!isTopInViewport || !isBottomInViewport)) {
          $video[0].pause();
        } else if ($video.prop("paused") && isTopInViewport && isBottomInViewport) {
          $video[0].play();
        }
      });
    }

    function initializeVideoPlayback() {
      if (!isInitialized) {
        handleVideoPlayback();
        isInitialized = true;
      }
    }

    $window.on("scroll resize", handleVideoPlayback);
    initializeVideoPlayback();
  });
  </script>
  <?php
}

// カスタム設定ページの追加
function avps_add_settings_page() {
  add_options_page(
    'Auto Video Play & Stop 設定',
    'Auto Video Play & Stop',
    'manage_options',
    'avps-settings',
    'avps_settings_page_callback',
    'dashicons-video-alt3'
  );
}
add_action('admin_menu', 'avps_add_settings_page');

// カスタム設定ページのコールバック関数
function avps_settings_page_callback() {
  if (!current_user_can('manage_options')) {
    return;
  }
  ?>
  <div class="wrap">
    <h1><?php echo esc_html(get_admin_page_title()); ?></h1>
    <form action="options.php" method="post">
      <?php
      settings_fields('avps_settings');
      do_settings_sections('avps_settings');
      submit_button('保存');
      ?>
    </form>
  </div>
  <script>
    document.getElementById("editableViewportThreshold").addEventListener("change", function() {
      var viewportPlayThreshold = document.getElementById("viewportPlayThreshold");
      var viewportStopThreshold = document.getElementById("viewportStopThreshold");
      if (this.checked) {
        viewportPlayThreshold.removeAttribute("readonly");
        viewportStopThreshold.removeAttribute("readonly");
      } else {
        viewportPlayThreshold.setAttribute("readonly", true);
        viewportStopThreshold.setAttribute("readonly", true);
      }
    });

    // 動画高さのしきい値からビューポートのしきい値を計算
    function updateViewportThresholds() {
      var visibleHeight = document.getElementById("visibleHeight").value;
      var visibleOffset = document.getElementById("visibleOffset").value;
      var viewportPlayThreshold = document.getElementById("viewportPlayThreshold");
      var viewportStopThreshold = document.getElementById("viewportStopThreshold");

      var calculatedPlayThreshold = (100 - visibleHeight) / 2;
      var calculatedStopThreshold = (100 - visibleOffset) / 2;

      viewportPlayThreshold.value = calculatedPlayThreshold;
      viewportStopThreshold.value = calculatedStopThreshold;
    }

    // 初期計算
    updateViewportThresholds();

    // 動画高さのしきい値が変更されたときに再計算
    document.getElementById("visibleHeight").addEventListener("input", updateViewportThresholds);
    document.getElementById("visibleOffset").addEventListener("input", updateViewportThresholds);
  </script>
  <?php
}

// 設定の登録
function avps_register_settings() {
  register_setting(
    'avps_settings',
    'avps_settings',
    'avps_settings_sanitize_callback'
  );

  add_settings_section(
    'avps_settings_section',
    '動画の再生・停止のしきい値設定',
    'avps_settings_section_callback',
    'avps_settings'
  );

  add_settings_field(
    'avps_visible_height',
    '再生するしきい値（動画高さの％）',
    'avps_visible_height_callback',
    'avps_settings',
    'avps_settings_section'
  );

  add_settings_field(
    'avps_visible_offset',
    '停止するしきい値（動画高さの％）',
    'avps_visible_offset_callback',
    'avps_settings',
    'avps_settings_section'
  );

  add_settings_field(
    'avps_editable_viewport_threshold',
    '手動で設定する',
    'avps_editable_viewport_threshold_callback',
    'avps_settings',
    'avps_settings_section'
  );

  add_settings_field(
    'avps_viewport_play_threshold',
    '再生するしきい値（ビューポート高さの％）',
    'avps_viewport_play_threshold_callback',
    'avps_settings',
    'avps_settings_section'
  );

  add_settings_field(
    'avps_viewport_stop_threshold',
    '停止するしきい値（ビューポート高さの％）',
    'avps_viewport_stop_threshold_callback',
    'avps_settings',
    'avps_settings_section'
  );
}
add_action('admin_init', 'avps_register_settings');

// セクションコールバック関数
function avps_settings_section_callback() {
  echo '<p>videoタグを使用した動画の自動再生、自動停止のしきい値を設定します。<br />※controls属性のついた動画は自動再生されません。<br />※動画制御のため、使用テーマ内で別途Jquery（3.6.0以上推奨）を読み込ませておく必要があります。</p>';
}

// 表示領域の高さフィールドのコールバック関数
function avps_visible_height_callback() {
  $settings = get_option('avps_settings');
  $value = isset($settings['avps_visible_height']) ? $settings['avps_visible_height'] : 50;
  echo '<input type="number" id="visibleHeight" min="0" max="100" step="1" name="avps_settings[avps_visible_height]" value="' . esc_attr($value) . '"> %';
}

// 表示領域のオフセットフィールドのコールバック関数
function avps_visible_offset_callback() {
  $settings = get_option('avps_settings');
  $value = isset($settings['avps_visible_offset']) ? $settings['avps_visible_offset'] : 50;
  echo '<input type="number" id="visibleOffset" min="0" max="100" step="1" name="avps_settings[avps_visible_offset]" value="' . esc_attr($value) . '"> %';
}

//手動で設定するのコールバック関数
function avps_editable_viewport_threshold_callback() {
  $settings = get_option('avps_settings');
  $isEditable = isset($settings['avps_editable_viewport_threshold']) && $settings['avps_editable_viewport_threshold'] === 'on';
  echo '<input type="checkbox" id="editableViewportThreshold" name="avps_settings[avps_editable_viewport_threshold]" ' . checked($isEditable, true, false) . '> 手動で設定する';
}

// ビューポートの高さの再生しきい値フィールドのコールバック関数
function avps_viewport_play_threshold_callback() {
  $settings = get_option('avps_settings');
  $value = isset($settings['avps_viewport_play_threshold']) ? $settings['avps_viewport_play_threshold'] : 50;
  $isEditable = isset($settings['avps_editable_viewport_threshold']) && $settings['avps_editable_viewport_threshold'] === 'on';
  echo '<input type="number" id="viewportPlayThreshold" min="0" max="100" step="1" name="avps_settings[avps_viewport_play_threshold]" value="' . esc_attr($value) . '" ' . ($isEditable ? '' : 'readonly') . '> %';
}

// ビューポートの高さの停止しきい値フィールドのコールバック関数
function avps_viewport_stop_threshold_callback() {
  $settings = get_option('avps_settings');
  $value = isset($settings['avps_viewport_stop_threshold']) ? $settings['avps_viewport_stop_threshold'] : 50;
  echo '<input type="number" id="viewportStopThreshold" min="0" max="100" step="1" name="avps_settings[avps_viewport_stop_threshold]" value="' . esc_attr($value) . '" readonly> %';
}

// サニタイズコールバック関数
function avps_settings_sanitize_callback($input) {
  $new_input = [];

  if (isset($input['avps_visible_height'])) {
    $new_input['avps_visible_height'] = intval($input['avps_visible_height']);
  }

  if (isset($input['avps_visible_offset'])) {
    $new_input['avps_visible_offset'] = intval($input['avps_visible_offset']);
  }

  if (isset($input['avps_viewport_play_threshold'])) {
    $new_input['avps_viewport_play_threshold'] = intval($input['avps_viewport_play_threshold']);
  }

  if (isset($input['avps_viewport_stop_threshold'])) {
    $new_input['avps_viewport_stop_threshold'] = intval($input['avps_viewport_stop_threshold']);
  }

  if (isset($input['avps_editable_viewport_threshold'])) {
    $new_input['avps_editable_viewport_threshold'] = sanitize_text_field($input['avps_editable_viewport_threshold']);
  }

  return $new_input;
}

//プラグイン一覧に「設定」のリンク追加
function avps_add_settings_link($links) {
    $settings_link = '<a href="options-general.php?page=avps-settings">' . __('設定') . '</a>';
    array_unshift($links, $settings_link);
    return $links;
}

$plugin = plugin_basename(__FILE__);
add_filter("plugin_action_links_$plugin", 'avps_add_settings_link');

?>
